@@ -7,6 +7,11 @@
 //! [`CSC`]: en.wikipedia.org/wiki/Sparse_matrix#Compressed_sparse_column_(CSC_or_CCS)
 //! [`CSR`]: en.wikipedia.org/wiki/Sparse_matrix#Compressed_sparse_row_(CSR,_CRS_or_Yale_format)
 
+#![allow(
+  // Serde
+  clippy::integer_arithmetic
+)]
+
 mod csl_error;
 mod csl_line_constructor;
 mod csl_line_iter;
@@ -150,7 +155,7 @@ where
 
     let innermost_dim_is_zero = {
       let mut iter = dims.iter().copied();
-      while let Some(dim) = iter.next() {
+      for dim in &mut iter {
         if dim != 0 {
           break;
         }
@@ -165,7 +170,7 @@ where
       return Err(CslError::DiffDataIndcsLength.into());
     }
 
-    if !are_in_ascending_order(&offs_ref, |a, b| [a, b]) {
+    if !are_in_ascending_order(offs_ref, |a, b| [a, b]) {
       return Err(CslError::InvalidOffsetsOrder.into());
     }
 
@@ -187,21 +192,27 @@ where
       }
     }
 
-    let first_off = if let Some(r) = offs_ref.first() {
+    let first_off = if let Some(r) = offs_ref.first().copied() {
       r
     } else {
       return Ok(Self { data, dims, indcs, offs });
     };
 
     if let Some(last_ref) = offs_ref.last() {
-      let last = last_ref - first_off;
-      if last != data_ref.len() || last != indcs_ref.len() {
-        return Err(CslError::LastOffsetDifferentNnz.into());
+      if let Some(last) = last_ref.checked_sub(first_off) {
+        if last != data_ref.len() || last != indcs_ref.len() {
+          return Err(CslError::LastOffsetDifferentNnz.into());
+        }
       }
     }
 
     let has_duplicated_indices = windows2(offs_ref).any(|[a, b]| {
-      if let Some(indcs) = indcs_ref.get(a - first_off..b - first_off) {
+      let fun = || {
+        let first = a.checked_sub(first_off)?;
+        let last = b.checked_sub(first_off)?;
+        indcs_ref.get(first..last)
+      };
+      if let Some(indcs) = fun() {
         has_duplicates(indcs)
       } else {
         false
@@ -574,7 +585,7 @@ where
     R: rand::Rng,
   {
     let mut csl = Csl { dims, ..Default::default() };
-    csl_rnd::CslRnd::new(&mut csl, nnz, rng)?.fill(cb)?;
+    csl_rnd::CslRnd::new(&mut csl, nnz, rng)?.fill(cb).ok_or(crate::Error::UnknownError)?;
     Self::new(csl.dims, csl.data, csl.indcs, csl.offs)
   }
 
